import { AngularFirestoreCollection, AngularFirestore, DocumentChangeAction, QueryFn, QueryGroupFn } from '@angular/fire/firestore';
import { EntityStore, EntityState, UpdateStateCallback, ActiveState, getEntityType } from '@datorama/akita';
import 'firebase/firestore';
import { CollectionOptions } from './collection.config';
import { WriteOptions } from '../utils/types';
import { Observable } from 'rxjs';
import { StoreOptions } from '../utils/store-options';
export declare type CollectionState<E = any> = EntityState<E, string> & ActiveState<string>;
export declare type orObservable<Input, Output> = Input extends Observable<infer I> ? Observable<Output> : Output;
export declare type DocOptions = {
    path: string;
} | {
    id: string;
};
export declare class CollectionService<S extends EntityState<any, string>> {
    protected store?: EntityStore<S>;
    private collectionPath?;
    private idsToListen;
    protected db: AngularFirestore;
    protected onCreate?(entity: getEntityType<S>, options: WriteOptions): any;
    protected onUpdate?(entity: Partial<getEntityType<S>>, options: WriteOptions): any;
    protected onDelete?(id: string, options: WriteOptions): any;
    constructor(store?: EntityStore<S>, collectionPath?: string);
    readonly idKey: string;
    /** The path to the collection in Firestore */
    readonly path: string | Observable<string>;
    /** A snapshot of the path */
    readonly currentPath: string;
    /** An observable version of the path */
    readonly path$: Observable<string>;
    /**
     * The Angular Fire collection
     * @notice If path is an observable, it becomes an observable.
     */
    readonly collection: AngularFirestoreCollection<getEntityType<S>>;
    /** Preformat the document before updating Firestore */
    protected preFormat<E extends getEntityType<S>>(document: Readonly<Partial<E>>): E;
    /** The config given by the `CollectonConfig` */
    readonly config: CollectionOptions;
    /**
     * Sync a store with a collection query of Firestore
     * @param path Path of the collection in Firestore
     * @param queryFn A query function to filter document of the collection
     * @param storeOptions Options about the store to sync with Firestore
     * @example
     * service.syncCollection({ storeName: 'movies-latest', loading: false }).subscribe();
     */
    syncCollection(storeOptions?: Partial<StoreOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    /**
     * @example
     * service.syncCollection(activePath$).subscribe();
     */
    syncCollection(path: string | Observable<string>, storeOptions?: Partial<StoreOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    /**
     * @example
     * service.syncCollection(ref => ref.limit(10), { storeName: 'movie-latest'}).subscribe();
     */
    syncCollection(query: QueryFn, storeOptions?: Partial<StoreOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    /**
     * @example
     * service.syncCollection('movies', ref => ref.limit(10), { loading: false }).subscribe();
     */
    syncCollection(path: string | Observable<string>, queryFn?: QueryFn, storeOptions?: Partial<StoreOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    /**
     * Sync a store with a collection group
     * @param collectionId An id of
     * @param queryFn A query function to filter document of the collection
     * @param storeOptions Options about the store to sync with Firestore
     */
    syncCollectionGroup(storeOptions?: Partial<StoreOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    syncCollectionGroup(queryGroupFn?: QueryGroupFn): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    syncCollectionGroup(collectionId: string, storeOptions?: Partial<StoreOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    syncCollectionGroup(collectionId: string, queryGroupFn?: QueryGroupFn, storeOptions?: Partial<StoreOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    /**
     * Sync the store with several documents
     * @param ids$ An array of ids or an observable of ids
     * @param storeOptions Options on the store to sync to
     */
    syncManyDocs(ids$: string[] | Observable<string[]>, storeOptions?: Partial<StoreOptions>): Observable<any[]>;
    /**
     * Stay in sync with one document
     * @param docOptions An object with EITHER `id` OR `path`.
     * @note We need to use id and path because there is no way to differentiate them.
     * @param storeOptions Options on the store to sync to
     */
    syncDoc(docOptions: DocOptions, storeOptions?: Partial<StoreOptions>): Observable<getEntityType<S>>;
    /**
     * Stay in sync with the active entity and set it active in the store
     * @param options A list of ids or An object with EITHER `id` OR `path`.
     * @note We need to use id and path because there is no way to differentiate them.
     * @param storeOptions Options on the store to sync to
     */
    syncActive(options: S['active'] extends any[] ? string[] : DocOptions, storeOptions?: Partial<StoreOptions>): S['active'] extends any[] ? Observable<getEntityType<S>[]> : Observable<getEntityType<S>>;
    /** Return the current value of the path from Firestore */
    getValue(id?: string): Promise<getEntityType<S>>;
    getValue(query?: QueryFn): Promise<getEntityType<S>[]>;
    /**
     * Add a document or a list of document to Firestore
     * @param docs A document or a list of document
     * @param write batch or transaction to run the operation into
     */
    add(documents: getEntityType<S> | getEntityType<S>[], options?: WriteOptions): Promise<any>;
    /**
     * Remove one or several document from Firestore
     * @param id A unique or list of id representing the document
     * @param write batch or transaction to run the operation into
     */
    remove(id: string | string[], options?: WriteOptions): Promise<void>;
    /**
     * Update one or several document in Firestore
     */
    update(entity: Partial<getEntityType<S>>, options?: WriteOptions): any;
    update(ids: string | string[], newStateFn: UpdateStateCallback<getEntityType<S>> | Partial<getEntityType<S>>, options?: WriteOptions): Promise<void>;
}
